@@ -358,6 +358,19 @@ PgSqlTaggedStatement tagged_statements[] = {
       "  FROM lease6_stat "
       "  WHERE subnet_id >= $1 and subnet_id <= $2 "
       "  ORDER BY subnet_id, lease_type, state" },
+
+    // TODO: remove single quotes from the following two SELECTs when the functions are implemented
+
+    // CHECK_LEASE4_LIMITS
+    { 1, { OID_TEXT },
+      "check_lease4_limits",
+      "SELECT 'checkLease4Limits($1)'" },
+
+    // CHECK_LEASE6_LIMITS
+    { 1, { OID_TEXT },
+      "check_lease6_limits",
+      "SELECT 'checkLease6Limits($1)'" },
+
     // End of list sentinel
     { 0,  { 0 }, NULL, NULL}
 };
@@ -2275,6 +2288,47 @@ PgSqlLeaseMgr::deleteExpiredReclaimedLeasesCommon(const uint32_t secs,
     return (deleteLeaseCommon(statement_index, bind_array));
 }
 
+string
+PgSqlLeaseMgr::checkLimits(ConstElementPtr const& user_context, StatementIndex const stindex) const {
+    // Bindings
+    PsqlBindArray bind_array;
+    std::string const user_context_str(user_context->str());
+    bind_array.add(user_context_str);
+
+    // Get a context.
+    PgSqlLeaseContextAlloc get_context(*this);
+    PgSqlLeaseContextPtr ctx(get_context.ctx_);
+
+    PgSqlResult r(PQexecPrepared(ctx->conn_,
+                                 tagged_statements[stindex].name,
+                                 tagged_statements[stindex].nbparams,
+                                 &bind_array.values_[0],
+                                 &bind_array.lengths_[0],
+                                 &bind_array.formats_[0], 0));
+
+    ctx->conn_.checkStatementError(r, tagged_statements[stindex]);
+
+    int rows = PQntuples(r);
+    if (rows > 1) {
+        isc_throw(MultipleRecords, "multiple records were found in the "
+                      "database where only one was expected for query "
+                      << tagged_statements[stindex].name);
+    }
+
+    std::string const limits(PgSqlExchange::getRawColumnValue(r, 0, 0));
+    return limits;
+}
+
+string
+PgSqlLeaseMgr::checkLimits4(ConstElementPtr const& user_context) const {
+    return checkLimits(user_context, CHECK_LEASE4_LIMITS);
+}
+
+string
+PgSqlLeaseMgr::checkLimits6(ConstElementPtr const& user_context) const {
+    return checkLimits(user_context, CHECK_LEASE6_LIMITS);
+}
+
 LeaseStatsQueryPtr
 PgSqlLeaseMgr::startLeaseStatsQuery4() {
     // Get a context
@@ -2371,6 +2425,11 @@ PgSqlLeaseMgr::wipeLeases6(const SubnetID& /*subnet_id*/) {
     isc_throw(NotImplemented, "wipeLeases6 is not implemented for PostgreSQL backend");
 }
 
+bool
+PgSqlLeaseMgr::isJsonSupported() const {
+    isc_throw(NotImplemented, "PgSqlLeaseMgr::isJsonSupported() not implemented");
+}
+
 std::string
 PgSqlLeaseMgr::getName() const {
     // Get a context
